@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,22 +14,26 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
 from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.cloud.dataflow_v1beta3.services.jobs_v1_beta3 import pagers
 from google.cloud.dataflow_v1beta3.types import environment
 from google.cloud.dataflow_v1beta3.types import jobs
@@ -219,6 +223,73 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -269,50 +340,22 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, JobsV1Beta3Transport):
             # transport is a JobsV1Beta3Transport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -324,6 +367,15 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -340,7 +392,7 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         self,
         request: Union[jobs.CreateJobRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> jobs.Job:
@@ -352,6 +404,25 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         Using ``projects.jobs.create`` is not recommended, as your job
         will always start in ``us-central1``.
 
+
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_create_job():
+                # Create a client
+                client = dataflow_v1beta3.JobsV1Beta3Client()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.CreateJobRequest(
+                )
+
+                # Make the request
+                response = client.create_job(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.dataflow_v1beta3.types.CreateJobRequest, dict]):
                 The request object. Request to create a Cloud Dataflow
@@ -365,7 +436,7 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         Returns:
             google.cloud.dataflow_v1beta3.types.Job:
                 Defines a job to be run by the Cloud
-                Dataflow service. nextID: 26
+                Dataflow service.
 
         """
         # Create or coerce a protobuf request object.
@@ -390,7 +461,7 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         self,
         request: Union[jobs.GetJobRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> jobs.Job:
@@ -402,6 +473,25 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         Using ``projects.jobs.get`` is not recommended, as you can only
         get the state of jobs that are running in ``us-central1``.
 
+
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_get_job():
+                # Create a client
+                client = dataflow_v1beta3.JobsV1Beta3Client()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.GetJobRequest(
+                )
+
+                # Make the request
+                response = client.get_job(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.dataflow_v1beta3.types.GetJobRequest, dict]):
                 The request object. Request to get the state of a Cloud
@@ -415,7 +505,7 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         Returns:
             google.cloud.dataflow_v1beta3.types.Job:
                 Defines a job to be run by the Cloud
-                Dataflow service. nextID: 26
+                Dataflow service.
 
         """
         # Create or coerce a protobuf request object.
@@ -440,7 +530,7 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         self,
         request: Union[jobs.UpdateJobRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> jobs.Job:
@@ -453,6 +543,25 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         only update the state of jobs that are running in
         ``us-central1``.
 
+
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_update_job():
+                # Create a client
+                client = dataflow_v1beta3.JobsV1Beta3Client()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.UpdateJobRequest(
+                )
+
+                # Make the request
+                response = client.update_job(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.dataflow_v1beta3.types.UpdateJobRequest, dict]):
                 The request object. Request to update a Cloud Dataflow
@@ -466,7 +575,7 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         Returns:
             google.cloud.dataflow_v1beta3.types.Job:
                 Defines a job to be run by the Cloud
-                Dataflow service. nextID: 26
+                Dataflow service.
 
         """
         # Create or coerce a protobuf request object.
@@ -491,7 +600,7 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         self,
         request: Union[jobs.ListJobsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListJobsPager:
@@ -505,6 +614,26 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         not recommended, as you can only get the list of jobs that are
         running in ``us-central1``.
 
+
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_list_jobs():
+                # Create a client
+                client = dataflow_v1beta3.JobsV1Beta3Client()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.ListJobsRequest(
+                )
+
+                # Make the request
+                page_result = client.list_jobs(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.dataflow_v1beta3.types.ListJobsRequest, dict]):
                 The request object. Request to list Cloud Dataflow jobs.
@@ -558,12 +687,31 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         self,
         request: Union[jobs.ListJobsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.AggregatedListJobsPager:
         r"""List the jobs of a project across all regions.
 
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_aggregated_list_jobs():
+                # Create a client
+                client = dataflow_v1beta3.JobsV1Beta3Client()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.ListJobsRequest(
+                )
+
+                # Make the request
+                page_result = client.aggregated_list_jobs(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.dataflow_v1beta3.types.ListJobsRequest, dict]):
                 The request object. Request to list Cloud Dataflow jobs.
@@ -617,13 +765,32 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         self,
         request: Union[jobs.CheckActiveJobsRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> jobs.CheckActiveJobsResponse:
         r"""Check for existence of active jobs in the given
         project across all regions.
 
+
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_check_active_jobs():
+                # Create a client
+                client = dataflow_v1beta3.JobsV1Beta3Client()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.CheckActiveJobsRequest(
+                )
+
+                # Make the request
+                response = client.check_active_jobs(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.dataflow_v1beta3.types.CheckActiveJobsRequest, dict]):
                 The request object. Request to check is active jobs
@@ -660,12 +827,30 @@ class JobsV1Beta3Client(metaclass=JobsV1Beta3ClientMeta):
         self,
         request: Union[jobs.SnapshotJobRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> snapshots.Snapshot:
         r"""Snapshot the state of a streaming job.
 
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_snapshot_job():
+                # Create a client
+                client = dataflow_v1beta3.JobsV1Beta3Client()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.SnapshotJobRequest(
+                )
+
+                # Make the request
+                response = client.snapshot_job(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.dataflow_v1beta3.types.SnapshotJobRequest, dict]):
                 The request object. Request to create a snapshot of a
