@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.cloud.dataflow_v1beta3.types import environment
 from google.cloud.dataflow_v1beta3.types import jobs
 from google.cloud.dataflow_v1beta3.types import templates
@@ -104,6 +109,42 @@ class TemplatesServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return TemplatesServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> TemplatesServiceTransport:
         """Returns the transport used by the client instance.
@@ -166,16 +207,35 @@ class TemplatesServiceAsyncClient:
 
     async def create_job_from_template(
         self,
-        request: templates.CreateJobFromTemplateRequest = None,
+        request: Union[templates.CreateJobFromTemplateRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> jobs.Job:
         r"""Creates a Cloud Dataflow job from a template.
 
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_create_job_from_template():
+                # Create a client
+                client = dataflow_v1beta3.TemplatesServiceClient()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.CreateJobFromTemplateRequest(
+                    gcs_path="gcs_path_value",
+                )
+
+                # Make the request
+                response = client.create_job_from_template(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.dataflow_v1beta3.types.CreateJobFromTemplateRequest`):
+            request (Union[google.cloud.dataflow_v1beta3.types.CreateJobFromTemplateRequest, dict]):
                 The request object. A request to create a Cloud Dataflow
                 job from a template.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -187,7 +247,7 @@ class TemplatesServiceAsyncClient:
         Returns:
             google.cloud.dataflow_v1beta3.types.Job:
                 Defines a job to be run by the Cloud
-                Dataflow service. nextID: 26
+                Dataflow service.
 
         """
         # Create or coerce a protobuf request object.
@@ -209,16 +269,35 @@ class TemplatesServiceAsyncClient:
 
     async def launch_template(
         self,
-        request: templates.LaunchTemplateRequest = None,
+        request: Union[templates.LaunchTemplateRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> templates.LaunchTemplateResponse:
         r"""Launch a template.
 
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_launch_template():
+                # Create a client
+                client = dataflow_v1beta3.TemplatesServiceClient()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.LaunchTemplateRequest(
+                    gcs_path="gcs_path_value",
+                )
+
+                # Make the request
+                response = client.launch_template(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.dataflow_v1beta3.types.LaunchTemplateRequest`):
+            request (Union[google.cloud.dataflow_v1beta3.types.LaunchTemplateRequest, dict]):
                 The request object. A request to launch a template.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
                 should be retried.
@@ -251,16 +330,35 @@ class TemplatesServiceAsyncClient:
 
     async def get_template(
         self,
-        request: templates.GetTemplateRequest = None,
+        request: Union[templates.GetTemplateRequest, dict] = None,
         *,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> templates.GetTemplateResponse:
         r"""Get the template associated with a template.
 
+        .. code-block:: python
+
+            from google.cloud import dataflow_v1beta3
+
+            def sample_get_template():
+                # Create a client
+                client = dataflow_v1beta3.TemplatesServiceClient()
+
+                # Initialize request argument(s)
+                request = dataflow_v1beta3.GetTemplateRequest(
+                    gcs_path="gcs_path_value",
+                )
+
+                # Make the request
+                response = client.get_template(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.dataflow_v1beta3.types.GetTemplateRequest`):
+            request (Union[google.cloud.dataflow_v1beta3.types.GetTemplateRequest, dict]):
                 The request object. A request to retrieve a Cloud
                 Dataflow job template.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
