@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,7 +15,6 @@
 #
 import os
 import mock
-import packaging.version
 
 import grpc
 from grpc.experimental import aio
@@ -36,9 +35,6 @@ from google.cloud.dataflow_v1beta3.services.jobs_v1_beta3 import JobsV1Beta3Asyn
 from google.cloud.dataflow_v1beta3.services.jobs_v1_beta3 import JobsV1Beta3Client
 from google.cloud.dataflow_v1beta3.services.jobs_v1_beta3 import pagers
 from google.cloud.dataflow_v1beta3.services.jobs_v1_beta3 import transports
-from google.cloud.dataflow_v1beta3.services.jobs_v1_beta3.transports.base import (
-    _GOOGLE_AUTH_VERSION,
-)
 from google.cloud.dataflow_v1beta3.types import environment
 from google.cloud.dataflow_v1beta3.types import jobs
 from google.cloud.dataflow_v1beta3.types import snapshots
@@ -50,20 +46,6 @@ from google.protobuf import timestamp_pb2  # type: ignore
 import google.auth
 
 
-# TODO(busunkim): Once google-auth >= 1.25.0 is required transitively
-# through google-api-core:
-# - Delete the auth "less than" test cases
-# - Delete these pytest markers (Make the "greater than or equal to" tests the default).
-requires_google_auth_lt_1_25_0 = pytest.mark.skipif(
-    packaging.version.parse(_GOOGLE_AUTH_VERSION) >= packaging.version.parse("1.25.0"),
-    reason="This test requires google-auth < 1.25.0",
-)
-requires_google_auth_gte_1_25_0 = pytest.mark.skipif(
-    packaging.version.parse(_GOOGLE_AUTH_VERSION) < packaging.version.parse("1.25.0"),
-    reason="This test requires google-auth >= 1.25.0",
-)
-
-
 def client_cert_source_callback():
     return b"cert bytes", b"key bytes"
 
@@ -211,7 +193,7 @@ def test_jobs_v1_beta3_client_client_options(
     options = client_options.ClientOptions(api_endpoint="squid.clam.whelk")
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(client_options=options)
+        client = client_class(transport=transport_name, client_options=options)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -228,7 +210,7 @@ def test_jobs_v1_beta3_client_client_options(
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class()
+            client = client_class(transport=transport_name)
             patched.assert_called_once_with(
                 credentials=None,
                 credentials_file=None,
@@ -245,7 +227,7 @@ def test_jobs_v1_beta3_client_client_options(
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class()
+            client = client_class(transport=transport_name)
             patched.assert_called_once_with(
                 credentials=None,
                 credentials_file=None,
@@ -261,20 +243,20 @@ def test_jobs_v1_beta3_client_client_options(
     # unsupported value.
     with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "Unsupported"}):
         with pytest.raises(MutualTLSChannelError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case GOOGLE_API_USE_CLIENT_CERTIFICATE has unsupported value.
     with mock.patch.dict(
         os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "Unsupported"}
     ):
         with pytest.raises(ValueError):
-            client = client_class()
+            client = client_class(transport=transport_name)
 
     # Check the case quota_project_id is provided
     options = client_options.ClientOptions(quota_project_id="octopus")
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -331,7 +313,7 @@ def test_jobs_v1_beta3_client_mtls_env_auto(
         )
         with mock.patch.object(transport_class, "__init__") as patched:
             patched.return_value = None
-            client = client_class(client_options=options)
+            client = client_class(client_options=options, transport=transport_name)
 
             if use_client_cert_env == "false":
                 expected_client_cert_source = None
@@ -373,7 +355,7 @@ def test_jobs_v1_beta3_client_mtls_env_auto(
                         expected_client_cert_source = client_cert_source_callback
 
                     patched.return_value = None
-                    client = client_class()
+                    client = client_class(transport=transport_name)
                     patched.assert_called_once_with(
                         credentials=None,
                         credentials_file=None,
@@ -395,7 +377,7 @@ def test_jobs_v1_beta3_client_mtls_env_auto(
                 return_value=False,
             ):
                 patched.return_value = None
-                client = client_class()
+                client = client_class(transport=transport_name)
                 patched.assert_called_once_with(
                     credentials=None,
                     credentials_file=None,
@@ -408,6 +390,83 @@ def test_jobs_v1_beta3_client_mtls_env_auto(
                 )
 
 
+@pytest.mark.parametrize("client_class", [JobsV1Beta3Client, JobsV1Beta3AsyncClient])
+@mock.patch.object(
+    JobsV1Beta3Client, "DEFAULT_ENDPOINT", modify_default_endpoint(JobsV1Beta3Client)
+)
+@mock.patch.object(
+    JobsV1Beta3AsyncClient,
+    "DEFAULT_ENDPOINT",
+    modify_default_endpoint(JobsV1Beta3AsyncClient),
+)
+def test_jobs_v1_beta3_client_get_mtls_endpoint_and_cert_source(client_class):
+    mock_client_cert_source = mock.Mock()
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "true".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source == mock_client_cert_source
+
+    # Test the case GOOGLE_API_USE_CLIENT_CERTIFICATE is "false".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "false"}):
+        mock_client_cert_source = mock.Mock()
+        mock_api_endpoint = "foo"
+        options = client_options.ClientOptions(
+            client_cert_source=mock_client_cert_source, api_endpoint=mock_api_endpoint
+        )
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source(
+            options
+        )
+        assert api_endpoint == mock_api_endpoint
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "never".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "never"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "always".
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_MTLS_ENDPOINT": "always"}):
+        api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+        assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+        assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert doesn't exist.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=False,
+        ):
+            api_endpoint, cert_source = client_class.get_mtls_endpoint_and_cert_source()
+            assert api_endpoint == client_class.DEFAULT_ENDPOINT
+            assert cert_source is None
+
+    # Test the case GOOGLE_API_USE_MTLS_ENDPOINT is "auto" and default cert exists.
+    with mock.patch.dict(os.environ, {"GOOGLE_API_USE_CLIENT_CERTIFICATE": "true"}):
+        with mock.patch(
+            "google.auth.transport.mtls.has_default_client_cert_source",
+            return_value=True,
+        ):
+            with mock.patch(
+                "google.auth.transport.mtls.default_client_cert_source",
+                return_value=mock_client_cert_source,
+            ):
+                (
+                    api_endpoint,
+                    cert_source,
+                ) = client_class.get_mtls_endpoint_and_cert_source()
+                assert api_endpoint == client_class.DEFAULT_MTLS_ENDPOINT
+                assert cert_source == mock_client_cert_source
+
+
 @pytest.mark.parametrize(
     "client_class,transport_class,transport_name",
     [
@@ -426,7 +485,7 @@ def test_jobs_v1_beta3_client_client_options_scopes(
     options = client_options.ClientOptions(scopes=["1", "2"],)
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file=None,
@@ -440,24 +499,26 @@ def test_jobs_v1_beta3_client_client_options_scopes(
 
 
 @pytest.mark.parametrize(
-    "client_class,transport_class,transport_name",
+    "client_class,transport_class,transport_name,grpc_helpers",
     [
-        (JobsV1Beta3Client, transports.JobsV1Beta3GrpcTransport, "grpc"),
+        (JobsV1Beta3Client, transports.JobsV1Beta3GrpcTransport, "grpc", grpc_helpers),
         (
             JobsV1Beta3AsyncClient,
             transports.JobsV1Beta3GrpcAsyncIOTransport,
             "grpc_asyncio",
+            grpc_helpers_async,
         ),
     ],
 )
 def test_jobs_v1_beta3_client_client_options_credentials_file(
-    client_class, transport_class, transport_name
+    client_class, transport_class, transport_name, grpc_helpers
 ):
     # Check the case credentials file is provided.
     options = client_options.ClientOptions(credentials_file="credentials.json")
+
     with mock.patch.object(transport_class, "__init__") as patched:
         patched.return_value = None
-        client = client_class(client_options=options)
+        client = client_class(client_options=options, transport=transport_name)
         patched.assert_called_once_with(
             credentials=None,
             credentials_file="credentials.json",
@@ -488,7 +549,74 @@ def test_jobs_v1_beta3_client_client_options_from_dict():
         )
 
 
-def test_create_job(transport: str = "grpc", request_type=jobs.CreateJobRequest):
+@pytest.mark.parametrize(
+    "client_class,transport_class,transport_name,grpc_helpers",
+    [
+        (JobsV1Beta3Client, transports.JobsV1Beta3GrpcTransport, "grpc", grpc_helpers),
+        (
+            JobsV1Beta3AsyncClient,
+            transports.JobsV1Beta3GrpcAsyncIOTransport,
+            "grpc_asyncio",
+            grpc_helpers_async,
+        ),
+    ],
+)
+def test_jobs_v1_beta3_client_create_channel_credentials_file(
+    client_class, transport_class, transport_name, grpc_helpers
+):
+    # Check the case credentials file is provided.
+    options = client_options.ClientOptions(credentials_file="credentials.json")
+
+    with mock.patch.object(transport_class, "__init__") as patched:
+        patched.return_value = None
+        client = client_class(client_options=options, transport=transport_name)
+        patched.assert_called_once_with(
+            credentials=None,
+            credentials_file="credentials.json",
+            host=client.DEFAULT_ENDPOINT,
+            scopes=None,
+            client_cert_source_for_mtls=None,
+            quota_project_id=None,
+            client_info=transports.base.DEFAULT_CLIENT_INFO,
+            always_use_jwt_access=True,
+        )
+
+    # test that the credentials from file are saved and used as the credentials.
+    with mock.patch.object(
+        google.auth, "load_credentials_from_file", autospec=True
+    ) as load_creds, mock.patch.object(
+        google.auth, "default", autospec=True
+    ) as adc, mock.patch.object(
+        grpc_helpers, "create_channel"
+    ) as create_channel:
+        creds = ga_credentials.AnonymousCredentials()
+        file_creds = ga_credentials.AnonymousCredentials()
+        load_creds.return_value = (file_creds, None)
+        adc.return_value = (creds, None)
+        client = client_class(client_options=options, transport=transport_name)
+        create_channel.assert_called_with(
+            "dataflow.googleapis.com:443",
+            credentials=file_creds,
+            credentials_file=None,
+            quota_project_id=None,
+            default_scopes=(
+                "https://www.googleapis.com/auth/cloud-platform",
+                "https://www.googleapis.com/auth/compute",
+                "https://www.googleapis.com/auth/compute.readonly",
+                "https://www.googleapis.com/auth/userinfo.email",
+            ),
+            scopes=None,
+            default_host="dataflow.googleapis.com",
+            ssl_credentials=None,
+            options=[
+                ("grpc.max_send_message_length", -1),
+                ("grpc.max_receive_message_length", -1),
+            ],
+        )
+
+
+@pytest.mark.parametrize("request_type", [jobs.CreateJobRequest, dict,])
+def test_create_job(request_type, transport: str = "grpc"):
     client = JobsV1Beta3Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -541,10 +669,6 @@ def test_create_job(transport: str = "grpc", request_type=jobs.CreateJobRequest)
     assert response.satisfies_pzs is True
 
 
-def test_create_job_from_dict():
-    test_create_job(request_type=dict)
-
-
 def test_create_job_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -623,7 +747,8 @@ async def test_create_job_async_from_dict():
     await test_create_job_async(request_type=dict)
 
 
-def test_get_job(transport: str = "grpc", request_type=jobs.GetJobRequest):
+@pytest.mark.parametrize("request_type", [jobs.GetJobRequest, dict,])
+def test_get_job(request_type, transport: str = "grpc"):
     client = JobsV1Beta3Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -676,10 +801,6 @@ def test_get_job(transport: str = "grpc", request_type=jobs.GetJobRequest):
     assert response.satisfies_pzs is True
 
 
-def test_get_job_from_dict():
-    test_get_job(request_type=dict)
-
-
 def test_get_job_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -758,7 +879,8 @@ async def test_get_job_async_from_dict():
     await test_get_job_async(request_type=dict)
 
 
-def test_update_job(transport: str = "grpc", request_type=jobs.UpdateJobRequest):
+@pytest.mark.parametrize("request_type", [jobs.UpdateJobRequest, dict,])
+def test_update_job(request_type, transport: str = "grpc"):
     client = JobsV1Beta3Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -811,10 +933,6 @@ def test_update_job(transport: str = "grpc", request_type=jobs.UpdateJobRequest)
     assert response.satisfies_pzs is True
 
 
-def test_update_job_from_dict():
-    test_update_job(request_type=dict)
-
-
 def test_update_job_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -893,7 +1011,8 @@ async def test_update_job_async_from_dict():
     await test_update_job_async(request_type=dict)
 
 
-def test_list_jobs(transport: str = "grpc", request_type=jobs.ListJobsRequest):
+@pytest.mark.parametrize("request_type", [jobs.ListJobsRequest, dict,])
+def test_list_jobs(request_type, transport: str = "grpc"):
     client = JobsV1Beta3Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -920,10 +1039,6 @@ def test_list_jobs(transport: str = "grpc", request_type=jobs.ListJobsRequest):
     assert response.next_page_token == "next_page_token_value"
 
 
-def test_list_jobs_from_dict():
-    test_list_jobs(request_type=dict)
-
-
 def test_list_jobs_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -974,8 +1089,10 @@ async def test_list_jobs_async_from_dict():
     await test_list_jobs_async(request_type=dict)
 
 
-def test_list_jobs_pager():
-    client = JobsV1Beta3Client(credentials=ga_credentials.AnonymousCredentials,)
+def test_list_jobs_pager(transport_name: str = "grpc"):
+    client = JobsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials, transport=transport_name,
+    )
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(type(client.transport.list_jobs), "__call__") as call:
@@ -1000,8 +1117,10 @@ def test_list_jobs_pager():
         assert all(isinstance(i, jobs.Job) for i in results)
 
 
-def test_list_jobs_pages():
-    client = JobsV1Beta3Client(credentials=ga_credentials.AnonymousCredentials,)
+def test_list_jobs_pages(transport_name: str = "grpc"):
+    client = JobsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials, transport=transport_name,
+    )
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(type(client.transport.list_jobs), "__call__") as call:
@@ -1073,9 +1192,8 @@ async def test_list_jobs_async_pages():
             assert page_.raw_page.next_page_token == token
 
 
-def test_aggregated_list_jobs(
-    transport: str = "grpc", request_type=jobs.ListJobsRequest
-):
+@pytest.mark.parametrize("request_type", [jobs.ListJobsRequest, dict,])
+def test_aggregated_list_jobs(request_type, transport: str = "grpc"):
     client = JobsV1Beta3Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1104,10 +1222,6 @@ def test_aggregated_list_jobs(
     assert response.next_page_token == "next_page_token_value"
 
 
-def test_aggregated_list_jobs_from_dict():
-    test_aggregated_list_jobs(request_type=dict)
-
-
 def test_aggregated_list_jobs_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1162,8 +1276,10 @@ async def test_aggregated_list_jobs_async_from_dict():
     await test_aggregated_list_jobs_async(request_type=dict)
 
 
-def test_aggregated_list_jobs_pager():
-    client = JobsV1Beta3Client(credentials=ga_credentials.AnonymousCredentials,)
+def test_aggregated_list_jobs_pager(transport_name: str = "grpc"):
+    client = JobsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials, transport=transport_name,
+    )
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1190,8 +1306,10 @@ def test_aggregated_list_jobs_pager():
         assert all(isinstance(i, jobs.Job) for i in results)
 
 
-def test_aggregated_list_jobs_pages():
-    client = JobsV1Beta3Client(credentials=ga_credentials.AnonymousCredentials,)
+def test_aggregated_list_jobs_pages(transport_name: str = "grpc"):
+    client = JobsV1Beta3Client(
+        credentials=ga_credentials.AnonymousCredentials, transport=transport_name,
+    )
 
     # Mock the actual call within the gRPC stub, and fake the request.
     with mock.patch.object(
@@ -1269,9 +1387,8 @@ async def test_aggregated_list_jobs_async_pages():
             assert page_.raw_page.next_page_token == token
 
 
-def test_check_active_jobs(
-    transport: str = "grpc", request_type=jobs.CheckActiveJobsRequest
-):
+@pytest.mark.parametrize("request_type", [jobs.CheckActiveJobsRequest, dict,])
+def test_check_active_jobs(request_type, transport: str = "grpc"):
     client = JobsV1Beta3Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1298,10 +1415,6 @@ def test_check_active_jobs(
     assert response.active_jobs_exist is True
 
 
-def test_check_active_jobs_from_dict():
-    test_check_active_jobs(request_type=dict)
-
-
 def test_check_active_jobs_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1356,7 +1469,8 @@ async def test_check_active_jobs_async_from_dict():
     await test_check_active_jobs_async(request_type=dict)
 
 
-def test_snapshot_job(transport: str = "grpc", request_type=jobs.SnapshotJobRequest):
+@pytest.mark.parametrize("request_type", [jobs.SnapshotJobRequest, dict,])
+def test_snapshot_job(request_type, transport: str = "grpc"):
     client = JobsV1Beta3Client(
         credentials=ga_credentials.AnonymousCredentials(), transport=transport,
     )
@@ -1395,10 +1509,6 @@ def test_snapshot_job(transport: str = "grpc", request_type=jobs.SnapshotJobRequ
     assert response.region == "region_value"
 
 
-def test_snapshot_job_from_dict():
-    test_snapshot_job(request_type=dict)
-
-
 def test_snapshot_job_empty_call():
     # This test is a coverage failsafe to make sure that totally empty calls,
     # i.e. request == None and no flattened fields passed, work.
@@ -1483,6 +1593,23 @@ def test_credentials_transport_error():
             transport=transport,
         )
 
+    # It is an error to provide an api_key and a transport instance.
+    transport = transports.JobsV1Beta3GrpcTransport(
+        credentials=ga_credentials.AnonymousCredentials(),
+    )
+    options = client_options.ClientOptions()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = JobsV1Beta3Client(client_options=options, transport=transport,)
+
+    # It is an error to provide an api_key and a credential.
+    options = mock.Mock()
+    options.api_key = "api_key"
+    with pytest.raises(ValueError):
+        client = JobsV1Beta3Client(
+            client_options=options, credentials=ga_credentials.AnonymousCredentials()
+        )
+
     # It is an error to provide scopes and a transport instance.
     transport = transports.JobsV1Beta3GrpcTransport(
         credentials=ga_credentials.AnonymousCredentials(),
@@ -1573,7 +1700,6 @@ def test_jobs_v1_beta3_base_transport():
         transport.close()
 
 
-@requires_google_auth_gte_1_25_0
 def test_jobs_v1_beta3_base_transport_with_credentials_file():
     # Instantiate the base transport with a credentials file
     with mock.patch.object(
@@ -1599,31 +1725,6 @@ def test_jobs_v1_beta3_base_transport_with_credentials_file():
         )
 
 
-@requires_google_auth_lt_1_25_0
-def test_jobs_v1_beta3_base_transport_with_credentials_file_old_google_auth():
-    # Instantiate the base transport with a credentials file
-    with mock.patch.object(
-        google.auth, "load_credentials_from_file", autospec=True
-    ) as load_creds, mock.patch(
-        "google.cloud.dataflow_v1beta3.services.jobs_v1_beta3.transports.JobsV1Beta3Transport._prep_wrapped_messages"
-    ) as Transport:
-        Transport.return_value = None
-        load_creds.return_value = (ga_credentials.AnonymousCredentials(), None)
-        transport = transports.JobsV1Beta3Transport(
-            credentials_file="credentials.json", quota_project_id="octopus",
-        )
-        load_creds.assert_called_once_with(
-            "credentials.json",
-            scopes=(
-                "https://www.googleapis.com/auth/cloud-platform",
-                "https://www.googleapis.com/auth/compute",
-                "https://www.googleapis.com/auth/compute.readonly",
-                "https://www.googleapis.com/auth/userinfo.email",
-            ),
-            quota_project_id="octopus",
-        )
-
-
 def test_jobs_v1_beta3_base_transport_with_adc():
     # Test the default credentials are used if credentials and credentials_file are None.
     with mock.patch.object(google.auth, "default", autospec=True) as adc, mock.patch(
@@ -1635,7 +1736,6 @@ def test_jobs_v1_beta3_base_transport_with_adc():
         adc.assert_called_once()
 
 
-@requires_google_auth_gte_1_25_0
 def test_jobs_v1_beta3_auth_adc():
     # If no credentials are provided, we should use ADC credentials.
     with mock.patch.object(google.auth, "default", autospec=True) as adc:
@@ -1653,28 +1753,10 @@ def test_jobs_v1_beta3_auth_adc():
         )
 
 
-@requires_google_auth_lt_1_25_0
-def test_jobs_v1_beta3_auth_adc_old_google_auth():
-    # If no credentials are provided, we should use ADC credentials.
-    with mock.patch.object(google.auth, "default", autospec=True) as adc:
-        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
-        JobsV1Beta3Client()
-        adc.assert_called_once_with(
-            scopes=(
-                "https://www.googleapis.com/auth/cloud-platform",
-                "https://www.googleapis.com/auth/compute",
-                "https://www.googleapis.com/auth/compute.readonly",
-                "https://www.googleapis.com/auth/userinfo.email",
-            ),
-            quota_project_id=None,
-        )
-
-
 @pytest.mark.parametrize(
     "transport_class",
     [transports.JobsV1Beta3GrpcTransport, transports.JobsV1Beta3GrpcAsyncIOTransport,],
 )
-@requires_google_auth_gte_1_25_0
 def test_jobs_v1_beta3_transport_auth_adc(transport_class):
     # If credentials and host are not provided, the transport class should use
     # ADC credentials.
@@ -1693,28 +1775,6 @@ def test_jobs_v1_beta3_transport_auth_adc(transport_class):
         )
 
 
-@pytest.mark.parametrize(
-    "transport_class",
-    [transports.JobsV1Beta3GrpcTransport, transports.JobsV1Beta3GrpcAsyncIOTransport,],
-)
-@requires_google_auth_lt_1_25_0
-def test_jobs_v1_beta3_transport_auth_adc_old_google_auth(transport_class):
-    # If credentials and host are not provided, the transport class should use
-    # ADC credentials.
-    with mock.patch.object(google.auth, "default", autospec=True) as adc:
-        adc.return_value = (ga_credentials.AnonymousCredentials(), None)
-        transport_class(quota_project_id="octopus")
-        adc.assert_called_once_with(
-            scopes=(
-                "https://www.googleapis.com/auth/cloud-platform",
-                "https://www.googleapis.com/auth/compute",
-                "https://www.googleapis.com/auth/compute.readonly",
-                "https://www.googleapis.com/auth/userinfo.email",
-            ),
-            quota_project_id="octopus",
-        )
-
-
 @pytest.mark.parametrize(
     "transport_class,grpc_helpers",
     [
@@ -2029,7 +2089,7 @@ def test_parse_common_location_path():
     assert expected == actual
 
 
-def test_client_withDEFAULT_CLIENT_INFO():
+def test_client_with_default_client_info():
     client_info = gapic_v1.client_info.ClientInfo()
 
     with mock.patch.object(
@@ -2094,3 +2154,33 @@ def test_client_ctx():
             with client:
                 pass
             close.assert_called()
+
+
+@pytest.mark.parametrize(
+    "client_class,transport_class",
+    [
+        (JobsV1Beta3Client, transports.JobsV1Beta3GrpcTransport),
+        (JobsV1Beta3AsyncClient, transports.JobsV1Beta3GrpcAsyncIOTransport),
+    ],
+)
+def test_api_key_credentials(client_class, transport_class):
+    with mock.patch.object(
+        google.auth._default, "get_api_key_credentials", create=True
+    ) as get_api_key_credentials:
+        mock_cred = mock.Mock()
+        get_api_key_credentials.return_value = mock_cred
+        options = client_options.ClientOptions()
+        options.api_key = "api_key"
+        with mock.patch.object(transport_class, "__init__") as patched:
+            patched.return_value = None
+            client = client_class(client_options=options)
+            patched.assert_called_once_with(
+                credentials=mock_cred,
+                credentials_file=None,
+                host=client.DEFAULT_ENDPOINT,
+                scopes=None,
+                client_cert_source_for_mtls=None,
+                quota_project_id=None,
+                client_info=transports.base.DEFAULT_CLIENT_INFO,
+                always_use_jwt_access=True,
+            )
